@@ -4,6 +4,7 @@
 # License: MIT
 
 import colorsys
+import itertools
 
 cssColors = {
 "purple":(128,0,128),
@@ -266,7 +267,7 @@ class Color():
         return Color(self.color[0], self.color[1]*factor, self.color[2], self.color[3])
 
     @property
-    def get_b(self):
+    def b(self):
         return self.color[2]
 
     def with_b(self, newval):
@@ -277,7 +278,7 @@ class Color():
         return Color(self.color[0], self.color[1], self.color[2]*factor, self.color[3])
 
     @property
-    def get_a(self):
+    def a(self):
         return self.color[3]
 
     def with_a(self, newval):
@@ -342,6 +343,22 @@ class Color():
         col = [x*(1-factor) + y*factor for x, y in zip(col1, col2)]
         return Color(*col)
 
+    def as_rgbstr(self):
+        return 'rgb({}, {}, {})'.format(int(self.color[0] * 255),
+                                       int(self.color[1] * 255),
+                                       int(self.color[2] * 255))
+
+    def as_rgb_bytes(self):
+        return (int(self.color[0] * 255),
+                int(self.color[1] * 255),
+                int(self.color[2] * 255))
+
+    def as_rgba_bytes(self):
+        return (int(self.color[0] * 255),
+                int(self.color[1] * 255),
+                int(self.color[2] * 255),
+                int(self.color[3] * 255))
+
     @staticmethod
     def clamp(v):
         try:
@@ -358,3 +375,45 @@ class Color():
             return self.color[i]
         else:
             raise IndexError()
+
+
+def make_colormap(length, colors, bands):
+    '''
+    Create a colormap, a list of varying colors.
+    :param length: Total size of list
+    :param colors: List of colors, must be at least 2 long.
+    :param bands: Relative size of each band. bands[i] gives the size of teh band between color[i] and color[i+1].
+                  len(bands) must be exactly 1 less than len(colors)
+    :return:
+    '''
+
+    # Check parameters
+    if length <= 0:
+        raise ValueError('length must be > 0')
+    if len(colors) < 2:
+        raise ValueError('colors list must have at least 2 elements')
+    if len(colors) != len(bands) + 1:
+        raise ValueError('colors list must be exactly 1 longer than bands list')
+
+
+    band_total = sum(bands)
+    band_breakpoints = [int(x*length/band_total) for x in itertools.accumulate(bands)]
+
+    current_colour = 0
+    band_index = 0
+    colormap = [None]*length
+    band_size = []
+    for i in range(length):
+        while band_breakpoints[current_colour] <= i:
+            band_size.append(band_index)
+            current_colour += 1
+            band_index = 0
+        colormap[i] = (current_colour, band_index)
+        band_index += 1
+    band_size.append(band_index)
+
+    colormap = [colors[col].lerp(colors[col+1], band/(band_size[col]-1))
+                 for col, band in colormap ]
+
+    return colormap
+
