@@ -14,6 +14,18 @@ TOP = 3
 BOTTOM = 4
 BASELINE = 5
 
+# Fill rule
+EVEN_ODD=0
+WINDING=1
+
+## Line cap/join
+MITER = 0  # join
+ROUND = 1  # join/cap
+BEVEL = 2  # join
+BUTT = 3   # cap
+SQUARE = 4 # cap
+
+
 
 def setup(ctx, pixel_width, pixel_height, width=None, height=None, startx=0, starty=0, background=None, flip=False):
     '''
