@@ -4,57 +4,119 @@
 # License: MIT
 import cairo
 import math
-
 from generativepy.drawing import LEFT, CENTER, RIGHT, BOTTOM, MIDDLE, BASELINE, TOP
+from generativepy.drawing import EVEN_ODD, WINDING
+from generativepy.drawing import MITER, ROUND, BEVEL, BUTT, SQUARE
+from generativepy.color import Color
 
 
 class Shape():
 
     def __init__(self, ctx):
         self.ctx = ctx
-        self.new_path = True
-        self.new_sub_path = True
+        self.extend = False
+        self.sub_path = False
+        self.final_close = False
+        self.added = False
 
-    def extend_path(self):
-        self.new_path = False
+    def extend_path(self, close=False):
+        self.sub_path = True
+        self.extend = True
+        self.final_close = close
+        return self
 
     def as_sub_path(self):
-        self.new_sub_path = False
+        self.sub_path = True
+        return self
 
     def _do_path_(self):
-        if self.new_path:
+        if not self.sub_path:
             self.ctx.new_path()
-        elif self.new_sub_path:
-            self.ctx.new_sub_path()
 
     def add(self):
         raise NotImplementedError()
 
-    def fill(self, color=None):
-        self.add()
-        if color:
-            self.ctx.set_source_rgba(*color)
-        self.ctx.fill()
+    def fill(self, color=Color(0), fill_rule=WINDING):
+        if not self.added:
+            self.add()
+            self.added = True
+        self.ctx.set_source_rgba(*color)
+
+        if fill_rule == WINDING:
+            self.ctx.set_fill_rule(cairo.FillRule.WINDING)
+        else:
+            self.ctx.set_fill_rule(cairo.FillRule.EVEN_ODD)
+
+        self.ctx.fill_preserve()
         return self
 
-    def stroke(self, color=None, line_width=None):
-        self.add()
-        if color:
-            self.ctx.set_source_rgba(*color)
-        if line_width != None:
-            self.ctx.set_line_width(line_width)
-        self.ctx.stroke()
+    def stroke(self, color=Color(0), line_width=1, dash=[], cap=SQUARE, join=MITER, miter_limit=None):
+        if not self.added:
+            self.add()
+            self.added = True
+
+        self.ctx.set_source_rgba(*color)
+
+        self.ctx.set_line_width(line_width)
+
+        self.ctx.set_dash(dash)
+
+        if cap == ROUND:
+            self.ctx.set_line_cap(cairo.LineCap.ROUND)
+        elif cap == BUTT:
+            self.ctx.set_line_cap(cairo.LineCap.BUTT)
+        else:
+            self.ctx.set_line_cap(cairo.LineCap.SQUARE)
+
+        if join == ROUND:
+            self.ctx.set_line_join(cairo.LineJoin.ROUND)
+        elif join == BEVEL:
+            self.ctx.set_line_join(cairo.LineJoin.BEVEL)
+        else:
+            self.ctx.set_line_join(cairo.LineJoin.MITER)
+
+        if miter_limit != None:
+            self.ctx.set_miter_limit(miter_limit)
+
+        self.ctx.stroke_preserve()
         return self
 
     def fill_stroke(self, fill_color, stroke_colour, line_width=1):
-        self.add()
-        self.ctx.set_source_rgba(*fill_color)
-        self.ctx.fill_preserve()
-        self.ctx.set_source_rgba(*stroke_colour)
-        self.ctx.set_line_width(line_width)
-        self.ctx.stroke()
+        self.fill(fill_color)
+        self.stroke(stroke_color, line_width)
         return self
 
+    def clip(self):
+        if not self.added:
+            self.add()
+            self.added = True
+        return self.ctx.clip_preserve()
+
+    def path(self):
+        if not self.added:
+            self.add()
+            self.added = True
+        return self.ctx.copy_path_flat()
+
+
+class Path(Shape):
+
+    def __init__(self, ctx):
+        super().__init__(ctx)
+        self.path = None
+        self.height = 0
+
+    def add(self):
+        self._do_path_()
+        if self.path:
+            self.ctx.append_path(self.path)
+        return self
+
+    def of(self, path):
+        self.path = path
+        return self
+
+
 class Rectangle(Shape):
 
     def __init__(self, ctx):
@@ -282,8 +344,11 @@ class Line(Shape):
 
     def add(self):
         self._do_path_()
-        self.ctx.move_to(*self.start)
+        if not self.extend:
+            self.ctx.move_to(*self.start)
         self.ctx.line_to(*self.end)
+        if self.final_close:
+            self.ctx.close_path()
         return self
 
     def of_start_end(self, start, end):
@@ -291,6 +356,11 @@ class Line(Shape):
         self.end = end
         return self
 
+    def of_end(self, end):
+        self.start = (0, 0)
+        self.end = end
+        return self
+
 
 def line(ctx, start, end):
     '''
@@ -303,6 +373,39 @@ def line(ctx, start, end):
     Line(ctx).of_start_end(start, end).add()
 
 
+class Bezier(Shape):
+
+    def __init__(self, ctx):
+        super().__init__(ctx)
+        self.a = (0, 0)
+        self.b = (0, 0)
+        self.c = (0, 0)
+        self.d = (0, 0)
+
+    def add(self):
+        self._do_path_()
+        if not self.extend:
+            self.ctx.move_to(*self.a)
+        self.ctx.curve_to(*self.b, *self.c, *self.d)
+        if self.final_close:
+            self.ctx.close_path()
+        return self
+
+    def of_abcd(self, a, b, c, d):
+        self.a = a
+        self.b = b
+        self.c = c
+        self.d = d
+        return self
+
+    def of_bcd(self, b, c, d):
+        self.a = (0, 0)
+        self.b = b
+        self.c = c
+        self.d = d
+        return self
+
+
 class Polygon(Shape):
 
     def __init__(self, ctx):
@@ -315,11 +418,15 @@ class Polygon(Shape):
         first = True
         for p in self.points:
             if first:
-                self.ctx.move_to(*p)
+                if not self.extend:
+                    self.ctx.move_to(*p)
                 first = False
             else:
-                self.ctx.line_to(*p)
-        if self.closed:
+                if len(p) == 6:
+                    self.ctx.curve_to(*p)
+                else:
+                    self.ctx.line_to(*p)
+        if self.closed or self.final_close:
             self.ctx.close_path()
         return self
 
@@ -371,6 +478,8 @@ class Circle(Shape):
             self.ctx.close_path()
         else:
             self.ctx.arc(*self.center, self.radius, self.start_angle, self.end_angle)
+            if self.final_close:
+                self.ctx.close_path()
         return self
 
     def of_center_radius(self, center, radius):
@@ -437,6 +546,8 @@ class Ellipse(Shape):
             self.ctx.close_path()
         else:
             self.ctx.arc(0, 0, self.radius_x, self.start_angle, self.end_angle)
+            if self.final_close:
+                self.ctx.close_path()
         self.ctx.restore()
         return self
 
